/*
 * RTC (real-time clock) support for various devices.
 *
 * Copyright (C) 2023, 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#ifdef __cplusplus

// Register locations.

enum Regs : unsigned
{
  Rtc_control                       = 0x000,      // RTCCR
  Rtc_seconds                       = 0x004,      // RTCSR
  Rtc_alarm_seconds                 = 0x008,      // RTCSAR
  Rtc_regulator                     = 0x00c,      // RTCGR

  Hibernate_control                 = 0x020,      // HCR
  Hibernate_wakeup_filter_counter   = 0x024,      // HWFCR
  Hibernate_reset_counter           = 0x028,      // HRCR
  Hibernate_wakeup_control          = 0x02c,      // HWCR
  Hibernate_wakeup_status           = 0x030,      // HWRSR
  Hibernate_scratch_pattern         = 0x034,      // HSPR
  Hibernate_write_enable_pattern    = 0x03c,      // WENR
  Hibernate_wakeup_pin_configure    = 0x048,      // WKUPPINCR
};

// Field definitions.

enum Control_bits : unsigned
{
  Control_write_ready               = 0x80,       // WRDY
  Control_1Hz                       = 0x40,       // 1HZ
  Control_1Hz_irq_enable            = 0x20,       // 1HZIE
  Control_alarm                     = 0x10,       // AF
  Control_alarm_irq_enable          = 0x08,       // AIE
  Control_alarm_enable              = 0x04,       // AE
  Control_external_divided          = 0x02,       // SELEXC (JZ4780)
  Control_rtc_enable                = 0x01,       // RTCE
};

enum Regulator_bits : unsigned
{
  Regulator_lock                    = 0x80000000, // LOCK
  Regulator_adjust_count_mask       = 0x03ff0000, // ADJC
  Regulator_1Hz_cycle_count_mask    = 0x0000ffff, // NC1HZ
};

enum Regulator_limits : unsigned
{
  Regulator_adjust_count_limit      = 0x03ff,     // ADJC
  Regulator_1Hz_cycle_count_limit   = 0xffff,     // NC1HZ
};

enum Regulator_shifts : unsigned
{
  Regulator_adjust_count_shift      = 16,         // ADJC
  Regulator_1Hz_cycle_count_shift   = 0,          // NC1HZ
};

enum Hibernate_control_bits : unsigned
{
  Hibernate_power_down              = 0x01,       // PD
};

enum Hibernate_wakeup_filter_counter_bits : unsigned
{
  Wakeup_minimum_time_mask          = 0xffe0,     // HWFCR
};

enum Hibernate_reset_counter_bits : unsigned
{
  Reset_assert_time_mask            = 0x7800,     // HRCR
};

enum Hibernate_wakeup_control_bits : unsigned
{
  Power_detect_enable_mask          = 0xfffffff8, // EPDET
  Rtc_alarm_wakeup_enable           = 0x00000001, // EALM
};

enum Hibernate_wakeup_status_bits : unsigned
{
  Accident_power_down               = 0x0100,     // APD
  Hibernate_reset                   = 0x0020,     // HR
  Pad_pin_reset                     = 0x0010,     // PPR
  Wakeup_pin_status                 = 0x0002,     // PIN
  Rtc_alarm_status                  = 0x0001,     // ALM
};

enum Hibernate_write_enable_pattern_bits : unsigned
{
  Write_enable_status               = 0x80000000, // WEN
  Write_enable_pattern_mask         = 0x0000ffff, // WENPAT
  Write_enable_pattern              = 0x0000a55a, // WENPAT
};

enum Hibernate_wakeup_pin_configure_bits : unsigned
{
  Rtc_oscillator_test_enable        = 0x00080000, // OSC_TE
  Oscillator_xtclk_rtclk            = 0x00040000, // OSC_RETON
  Oscillator_xtclk_low              = 0x00000000, // OSC_RETON
  Rtc_internal_oscillator_enable    = 0x00010000, // OSC_EN
  Wakeup_pin_extended_press_mask    = 0x000000f0, // P_JUD_LEN
  Wakeup_pin_extended_press_enable  = 0x0000000f, // P_RST_LEN
};

#endif /* __cplusplus */
